import org.eclipse.swt.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;

/**
 * @author Thomas Singer
 */
public class VirtualOwnerDrawnTable {

	// Constants ==============================================================

	static final int COUNT = 1000000;

	private static boolean packPendent = true;

	// Static =================================================================

	public static void main(String[] args) {
		final Display display = new Display();
		final Shell shell = new Shell(display);
		shell.setLayout(new FillLayout());

		final Table table = new Table(shell, SWT.VIRTUAL | SWT.BORDER | SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
		table.setHeaderVisible(true);
		final Listener listener = new Listener() {
			@Override
			public void handleEvent(Event event) {
				final TableItem item = (TableItem)event.item;
				if (event.type == SWT.EraseItem) {
					if (event.index < 2) {
						event.detail &= ~SWT.FOREGROUND;
					}
				}
				else if (event.type == SWT.PaintItem) {
					if (event.index == 0) {
						event.gc.drawText((String)item.getData(), event.x, event.y, true);
					}
					else if (event.index == 1) {
						final String text = (String)item.getData();
						final Point size = event.gc.textExtent(text);
						final Rectangle bounds = item.getBounds(event.index);
						event.gc.drawText(text, bounds.x + bounds.width - size.x, bounds.y + bounds.height - size.y, true);
					}
				}
				else if (event.type == SWT.MeasureItem) {
					final String text = (String)item.getData();
					final Point point = event.gc.textExtent(text);
					event.height = point.y * 2;
					event.width = Math.max(event.width, point.x);
				}
				else if (event.type == SWT.SetData) {
					final int index = table.indexOf(item);
					final String data = "Item " + index;
					item.setText(2, data);
					item.setData(data);

					if (packPendent) {
						packPendent = false;
						display.asyncExec(new Runnable() {
							@Override
							public void run() {
								table.setRedraw(false);
								for (TableColumn column : table.getColumns()) {
									column.pack();
								}
								table.setRedraw(true);
							}
						});
					}
				}
			}
		};
		table.addListener(SWT.EraseItem, listener);
		table.addListener(SWT.SetData, listener);
		table.addListener(SWT.MeasureItem, listener);
		table.addListener(SWT.PaintItem, listener);

		createColumn(SWT.LEFT, "OD Left", table);
		createColumn(SWT.RIGHT, "OD Right", table);
		createColumn(SWT.LEFT, "Primitive", table);

		table.setItemCount(COUNT);
		table.setTopIndex(50000);

		shell.setSize(400, 500);
		shell.open();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}
		display.dispose();
	}

	private static void createColumn(int style, String text, Table table) {
		final TableColumn tableColumn = new TableColumn(table, style);
		tableColumn.setText(text);
		tableColumn.setMoveable(true);
	}
}
